"""
Module to provide tests related to the MD011 rule.
"""
from test.markdown_scanner import MarkdownScanner

import pytest


@pytest.mark.rules
def test_md011_good_no_reversed():
    """
    Test to make sure this rule does not trigger with a document that
    contains no reversed links.
    """

    # Arrange
    scanner = MarkdownScanner()
    supplied_arguments = [
        "scan",
        "test/resources/rules/md011/good_no_reversed.md",
    ]

    expected_return_code = 0
    expected_output = ""
    expected_error = ""

    # Act
    execute_results = scanner.invoke_main(arguments=supplied_arguments)

    # Assert
    execute_results.assert_results(
        expected_output, expected_error, expected_return_code
    )


@pytest.mark.rules
def test_md011_bad_with_reversed():
    """
    Test to make sure this rule does trigger with a document that
    contains at least one reversed link.
    """

    # Arrange
    scanner = MarkdownScanner()
    supplied_arguments = [
        "scan",
        "test/resources/rules/md011/bad_with_reversed.md",
    ]

    expected_return_code = 1
    expected_output = (
        "test/resources/rules/md011/bad_with_reversed.md:2:8: "
        + "MD011: Reversed link syntax "
        + "[(reversed)[link]] (no-reversed-links)"
    )
    expected_error = ""

    # Act
    execute_results = scanner.invoke_main(arguments=supplied_arguments)

    # Assert
    execute_results.assert_results(
        expected_output, expected_error, expected_return_code
    )


@pytest.mark.rules
def test_md011_good_markdown_footnote():
    """
    Test to make sure this rule does not trigger with a document that
    contains what looks like a reversed link, but also looks like a footnote.
    """

    # Arrange
    scanner = MarkdownScanner()
    supplied_arguments = [
        "scan",
        "test/resources/rules/md011/good_markdown_extra.md",
    ]

    expected_return_code = 0
    expected_output = ""
    expected_error = ""

    # Act
    execute_results = scanner.invoke_main(arguments=supplied_arguments)

    # Assert
    execute_results.assert_results(
        expected_output, expected_error, expected_return_code
    )


@pytest.mark.rules
def test_md011_good_with_reversed_in_code_block():
    """
    Test to make sure this rule does not trigger with a document that
    contains a "reversed link" in a code block.
    """

    # Arrange
    scanner = MarkdownScanner()
    supplied_arguments = [
        "scan",
        "test/resources/rules/md011/good_with_reversed_in_code_block.md",
    ]

    expected_return_code = 0
    expected_output = ""
    expected_error = ""

    # Act
    execute_results = scanner.invoke_main(arguments=supplied_arguments)

    # Assert
    execute_results.assert_results(
        expected_output, expected_error, expected_return_code
    )


@pytest.mark.rules
def test_md011_good_with_reversed_in_html_block():
    """
    Test to make sure this rule does not trigger with a document that
    contains a "reversed link" in a HTML block.
    """

    # Arrange
    scanner = MarkdownScanner()
    supplied_arguments = [
        "scan",
        "test/resources/rules/md011/good_with_reversed_in_html_block.md",
    ]

    expected_return_code = 0
    expected_output = ""
    expected_error = ""

    # Act
    execute_results = scanner.invoke_main(arguments=supplied_arguments)

    # Assert
    execute_results.assert_results(
        expected_output, expected_error, expected_return_code
    )
